import React from 'react';
import { selectAll, select } from 'd3';
import PropTypes from 'prop-types';
import { VennDiagram } from 'venn.js';

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

var Venn = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(Venn, _React$Component);

  function Venn() {
    var _this;

    _this = _React$Component.apply(this, arguments) || this;
    _this.state = {
      selectors: [],
      sets: []
    };

    _this.fillVenn = function () {
      // @ts-ignore
      for (var _iterator = _createForOfIteratorHelperLoose(_this.props.selectors), _step; !(_step = _iterator()).done;) {
        var selector = _step.value;
        var relationSelector = selector.replace(String.fromCharCode(92), String.fromCharCode(92, 92));
        var select = "g[venn-area-part-id='" + relationSelector + "']"; // @ts-ignore

        var node = _this.state.svg.select(select).node();

        if (node) {
          _this.selectNode(node);
        }
      }
    };

    return _this;
  }

  var _proto = Venn.prototype;

  _proto.getIntersectionAreasMapping = function getIntersectionAreasMapping() {
    var intersectionAreasMapping = {};
    var vennAreas = selectAll(".venn-area");
    vennAreas.each(function (areaData, areaIdx, areas) {
      var area = areas[areaIdx];
      var areaSets = areaData.sets;
      var areaSelection = select(area);
      var areaD = areaSelection.select("path").attr("d");
      var areaSetsId = area.dataset.vennSets;
      var intersectedAreas = selectAll(".venn-area").filter(function (cAreaData, cAreaIdx, cAreas) {
        // @ts-ignore
        var cAreaSetsId = cAreas[cAreaIdx].dataset.vennSets;
        var cAreaSets = cAreaData.sets; // @ts-ignore

        var isContained = areaSets.every(function (setId) {
          return cAreaSets.indexOf(setId) > -1;
        });
        return isContained && cAreaSetsId !== areaSetsId;
      }).nodes().map(function (intersectedArea) {
        var intersectedAreaSelection = select(intersectedArea);
        return {
          sets: intersectedAreaSelection.data()[0].sets,
          d: intersectedAreaSelection.select("path").attr("d")
        };
      });
      intersectionAreasMapping[areaSetsId] = {
        vennArea: {
          sets: areaSets,
          d: areaD
        },
        intersectedAreas: intersectedAreas
      };
    });
    return intersectionAreasMapping;
  };

  _proto.appendVennAreaParts = function appendVennAreaParts(svg, intersectionAreasMapping) {
    for (var areaSetsId in intersectionAreasMapping) {
      var intersectionAreasItem = intersectionAreasMapping[areaSetsId];
      var vennArea = intersectionAreasItem.vennArea;
      var intersectedAreas = intersectionAreasItem.intersectedAreas;
      var partId = this.getPartId(vennArea, intersectedAreas);
      var d = [vennArea.d].concat(intersectedAreas.map(function (intersectedArea) {
        return intersectedArea.d;
      }));
      this.appendVennAreaPart(svg, d.join(""), partId);
    }
  };

  _proto.appendLabels = function appendLabels(svg, labels) {
    // @ts-ignore
    labels.nodes().forEach(function (label) {
      // @ts-ignore
      svg.append(function () {
        return label;
      });
    });
  };

  _proto.appendVennAreaPart = function appendVennAreaPart(svg, d, partId) {
    svg.append("g").attr("class", "venn-area-part").attr("venn-area-part-id", partId).append("path").attr("d", d).attr("fill-rule", "evenodd");
  };

  _proto.appendPatterns = function appendPatterns(defs) {
    var colors = ["none", "#009fdf"];
    colors.forEach(function (color, idx) {
      var diagonal = defs.append("pattern").attr("id", "diagonal" + idx).attr("patternUnits", "userSpaceOnUse").attr("width", "10").attr("height", "10");
      diagonal.append('rect').attr("width", "10").attr("height", "10").attr("x", "0").attr("y", "0").attr("fill", color).attr("fill-opacity", "0.15");
      diagonal.append("path").attr("d", "M-1,1 l2,-2 M0,10 l10,-10 M9,11 l2,-2").attr("stroke", "#000000").attr("opacity", "1").attr("stroke-width", "1");
    });
  };

  _proto.getPartId = function getPartId(vennArea, intersectedAreas) {
    var partId = "(" + vennArea.sets.join("∩") + ")";
    partId += intersectedAreas.length > 1 ? "\\(" : "";
    partId += intersectedAreas.length === 1 ? "\\" : "";
    partId += intersectedAreas.map(function (intersectedArea) {
      return intersectedArea.sets;
    }).map(function (set) {
      return "(" + set.join("∩") + ")";
    }).join("∪");
    partId += intersectedAreas.length > 1 ? ")" : "";
    return partId;
  };

  _proto.bindVennAreaPartListeners = function bindVennAreaPartListeners(div) {
    div.selectAll("g").on("mouseover", function (d, i) {
      // @ts-ignore
      var node = select(this);
      var nodePath = node.select("path");
      var nodeAlreadySelected = node.classed("selected");
      nodePath.attr("style", nodeAlreadySelected ? "fill: url(#diagonal1)" : "fill: #009fdf; fill-opacity: 0.15");
    }).on("mouseout", function (d, i) {
      // @ts-ignore
      var node = select(this);
      var nodePath = node.select("path");
      var nodeAlreadySelected = node.classed("selected");
      nodePath.attr("style", nodeAlreadySelected ? "fill: url(#diagonal0)" : "fill: #ffffff");
    }).on("click", function (d, i) {
      // @ts-ignore
      var node = select(this);
      var nodePath = node.select("path");
      var nodeAlreadySelected = node.classed("selected");
      var nodePathStyle = !nodeAlreadySelected ? "fill: url(#diagonal1)" : "fill: #ffffff";
      nodePath.attr("style", nodePathStyle);
      node.classed("selected", !nodeAlreadySelected);
    });
  };

  _proto.removeOriginalVennAreas = function removeOriginalVennAreas() {
    selectAll("g.venn-area").remove();
  };

  _proto.clear = function clear() {
    selectAll("g").classed("selected", false);
    selectAll("path").attr("style", "fill: #ffffff");
  };

  _proto.selectNode = function selectNode(DOMnode) {
    var node = select(DOMnode);
    var nodePath = node.select("path");
    var nodeAlreadySelected = node.classed("selected");
    var nodePathStyle = !nodeAlreadySelected ? "fill: url(#diagonal0)" : "fill: #ffffff";
    nodePath.attr("style", nodePathStyle);
    node.classed("selected", !nodeAlreadySelected);
  };

  _proto.componentDidMount = function componentDidMount() {
    var _this2 = this;

    var chart = VennDiagram(); // @ts-ignore

    var div = select(this.div).datum(this.props.sets).call(chart);
    var svg = div.select("svg");
    var defs = svg.append("defs");
    var labels = div.selectAll("text").remove();
    this.appendPatterns(defs); // @ts-ignore

    this.intersectionAreasMapping = this.getIntersectionAreasMapping(); // @ts-ignore

    this.appendVennAreaParts(svg, this.intersectionAreasMapping); // @ts-ignore

    this.appendLabels(svg, labels);
    this.bindVennAreaPartListeners(div);
    this.removeOriginalVennAreas();
    this.setState({
      svg: svg
    }, function () {
      // @ts-ignore
      if (_this2.props.selectors) {
        _this2.fillVenn();
      }
    });
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
    // @ts-ignore
    if (this.props.selectors !== prevProps.selectors && this.props.selectors) {
      this.clear();
      this.fillVenn();
    }
  };

  _proto.render = function render() {
    var _this3 = this;

    return (// @ts-ignore
      React.createElement("div", {
        id: "venn",
        ref: function ref(div) {
          return _this3.div = div;
        }
      })
    );
  };

  return Venn;
}(React.Component);
Venn.propTypes = {
  selectors: /*#__PURE__*/PropTypes.arrayOf(PropTypes.string),
  sets: /*#__PURE__*/PropTypes.arrayOf( /*#__PURE__*/PropTypes.shape({
    label: PropTypes.string,
    sets: /*#__PURE__*/PropTypes.arrayOf(PropTypes.string).isRequired,
    size: PropTypes.number.isRequired
  }))
};

export { Venn };
//# sourceMappingURL=react-venn-selectable.esm.js.map
